# $Id: Auth.pm,v 1.2 2011/12/11 07:26:49 koba Exp $
#	Copyright(c) 2006-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::Auth;

use strict;
use XiuMai;
use XiuMai::Util qw(randstr);
use FileHandle;
use DirHandle;
use vars qw($VERSION $EXPDATE);

$VERSION = $XiuMai::VERSION;

$EXPDATE = 14;

sub login {
	my ($login_name, $passwd) = @_;

	my $passwd_file = "$XiuMai::HOME/auth/local/passwd";
	my $fh = new FileHandle($passwd_file)	or die "$passwd_file: $!";
	while (<$fh>) {
		chomp;
		my ($login, $crypt) = split(/:/);
		next if ($login ne $login_name);
		return (crypt($passwd, $crypt) eq $crypt);
	}
	$fh->close;

	return;
}

sub get_login_name {
	my ($cookie) = @_;

	return if (! defined $cookie);

	my $cookie_file = "$XiuMai::HOME/auth/cookie/$cookie";
	$cookie_file =~ /^(.*)$/	and $cookie_file = $1;

	my $fh = new FileHandle($cookie_file)	or return;
	chomp(my $login_name = <$fh>);
	$fh->close;

	my $now = time;
	utime($now, $now, $cookie_file)	or die "$cookie_file: $!";

	$login_name =~ /(.*)/	and $login_name = $1;

	return $login_name;
}

sub make_auth_cookie {
	my ($login_name) = @_;

	my $cookie;
	my $cookie_dir = "$XiuMai::HOME/auth/cookie";

	my $dh = new DirHandle($cookie_dir)			or die "$cookie_dir: $!";
	while ($cookie = $dh->read) {
		next if ($cookie =~ /^\./);
		my $cookie_file = "$cookie_dir/$cookie";
		my $fh = new FileHandle($cookie_file)	or die "$cookie_file: $!";
		chomp(my $login = <$fh>);
		$fh->close;
		return $cookie	if ($login eq $login_name);
	}
	$dh->close;

	for (;;) {
		$cookie = randstr(32);
		last if (! -f "$cookie_dir/$cookie");
	}

	my $cookie_file = "$cookie_dir/$cookie";
	my $fh = new FileHandle(">$cookie_file")	or die "$cookie_file: $!";
	print $fh "$login_name\n";
	$fh->close;

	return $cookie;
}

sub delete_auth_cookie {
	my ($cookie) = @_;

	$cookie =~ /^(.*)$/	and $cookie = $1;

	unlink("$XiuMai::HOME/auth/cookie/$cookie");
}

sub cleanup_auth_cookie {

	my $cookie_dir = "$XiuMai::HOME/auth/cookie";

	my $dh = new DirHandle($cookie_dir)			or die "$cookie_dir: $!";
	while (my $cookie = $dh->read) {
		next if ($cookie =~ /^\./);
		$cookie =~ /^(.*)$/	and $cookie = $1;
		my $cookie_file = "$cookie_dir/$cookie";
		next if (time < (stat($cookie_file))[9] + $EXPDATE*60*60*24);
		unlink($cookie_file)	or die "$cookie_file: $!";
	}
}

1;
