# $Id: Module.pm,v 1.5 2011/12/17 13:26:25 koba Exp $
#	Copyright(c) 2007-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::Module;

use strict;
use XiuMai::Util qw(cdata canonpath);
use vars qw($VERSION);

$VERSION = do { my @r = (q$Revision: 1.5 $ =~ /\d+/g);
				sprintf("%d." . "%02d" x $#r, @r) };

sub new {
	my $class = shift;
	my ($text) = @_;

	{
		no strict 'refs';
		my %inline = map { $_ => 0 } @{"$class\::INLINE_MODULE"};
		my %block  = map { $_ => 0 } @{"$class\::BLOCK_MODULE"};
		foreach (@{"$class\::NP_MODULE"}) {
			$inline{$_} = 1		if (exists $inline{$_});
			$block{$_}  = 1		if (exists $block{$_});
		}

		return bless {
					inline => \%inline,
					block  => \%block,
					Text   => $text,
				}, $class;
	}
}

sub text {
	my $self = shift;
	return $self->{Text};
}

sub resource {
	my $self = shift;
	return $self->text->resource;
}

sub request {
	my $self = shift;
	return $self->text->request;
}

sub _url {
	my $self = shift;
	my ($url) = @_;
	return $url =~ m|^/|
				? ($self->request ? $self->request->xiumai_base_url : '')
					. canonpath($url)
				: $url;
}

sub dispatch {
	my $self = shift;
	my ($mode, $name, $param, $value) = @_;
	return if (! exists $self->{$mode}->{$name});

	if ($mode eq 'inline' && defined $value) {
		$value = $self->{$mode}->{$name}
					? cdata($value)
					: $self->text->inline($value);
	}
	elsif ($mode eq 'block' && defined $value) {
		$value = $self->{$mode}->{$name}
					? $self->text->np_block($value)
					: $self->text->block($value);
	}
	return $self->$name($mode, $param, $value);
}

1;
