# $Id: RCS.pm,v 1.4 2012/10/21 11:02:21 koba Exp $
#	Copyright(c) 2012 Satoshi Kobayashi, All rights reserved.

package XiuMai::RCS;

use strict;
use XiuMai;
use FileHandle;
use vars qw($VERSION $BINDIR $RCSDIR);

$VERSION = $XiuMai::VERSION;

$BINDIR = '/usr/bin';
$RCSDIR = '.ARCH';

BEGIN {
	$ENV{PATH} = '/usr/bin:/bin';
	$ENV{LOGNAME} = 'xiumai';
}

sub mkdir {
	my ($dir) = @_;
	return 	if (! $BINDIR);
	return 	if (-d "$dir/$RCSDIR");
	mkdir("$dir/$RCSDIR", 0777)		or die "mkdir $dir/$RCSDIR: $!";
}

sub rmdir {
	my ($dir) = @_;
	return 	if (! -d "$dir/$RCSDIR");
	rmdir("$dir/$RCSDIR")			or die "rmdir $dir/$RCSDIR: $!";
}

sub new {
	my $class = shift;
	my ($src_file) = @_;

	return	if (! -f $src_file);

	my ($dir, $file) = $src_file =~ m|^(.*)/([^/]+)$|	or return;
	my $rcs_file = "$dir/$RCSDIR/$file,v";

	return bless {
		src_file => $src_file,
		rcs_file => $rcs_file,
	}, $class;
}

sub checkin {
	my $self = shift;
	my ($auther, $locker) = @_;

	return 	if (! $BINDIR);

	$locker = $auther	if (! defined $locker);

	my $mtime = (stat($self->{src_file}))[9];
	my $size  = -s $self->{src_file};
	my $message = join("\t", ($mtime, $size, $auther, $locker));

	system("$BINDIR/ci", '-q', '-d', '-t-', "-w$auther", "-m$message",
				$self->{src_file}, $self->{rcs_file}) == 0
			or die "system $BINDIR/ci: $?";

	system("$BINDIR/co", '-l', '-q', '-M',
				$self->{src_file}, $self->{rcs_file}) == 0
			or die "system $BINDIR/ci: $?";
}

sub log {
	my $self = shift;

	return 	if (! $BINDIR);
	return	if (! -f $self->{rcs_file});

	my $fh = new FileHandle('-|')
				or exec("$BINDIR/rlog", $self->{rcs_file})
				or die "exec $BINDIR/rlog: $!";
	my (@log, $log, $stat);
	while (<$fh>) {
		chomp;
		if (! defined $stat) {
			$stat = 'revision'		if (/^\-+$/);
		}
		elsif ($stat eq 'revision') {
			$log = {};
			($log->{rev})
				= /^revision (\d+(?:\.\d+)+)/;
			$stat = 'date';
		}
		elsif ($stat eq 'date') {
			$stat = 'message';
		}
		elsif ($stat eq 'message') {
			($log->{mtime}, $log->{size}, $log->{author}, $log->{locker})
				= split(/\t/);
			$log->{author} =~ /(.*)/	and $log->{author} = $1;
			$log->{locker} =~ /(.*)/	and $log->{locker} = $1;
			push(@log, $log);
			$stat = undef;
		}
	}
	$fh->close;

	return wantarray ? @log : $log[0];
}

sub diff {
	my $self = shift;
	my ($rev1, $rev2)
		= map { /^(\d+(?:\.\d+)+)$/ and $1 } @_;

	my $diff;
	my $fh;
	if (defined $rev1 && defined $rev2) {
		$fh = new FileHandle('-|')
				or exec("$BINDIR/rcsdiff", '-q', '-c',
						"-r$rev1", "-r$rev2", $self->{rcs_file})
				or die "exec $BINDIR/rcsdiff: $!";
	}
	elsif (defined $rev1) {
		$fh = new FileHandle('-|')
				or exec("$BINDIR/rcsdiff", '-q', '-c',
						"-r$rev1", $self->{src_file}, $self->{rcs_file})
				or die "exec $BINDIR/rcsdiff: $!";
	}
	else {
		return '';
	}
	my $i;
	while (<$fh>) {
		$i++;
		next if ($i <= 3);
		$diff .= $_;
	}
	$fh->close;
	return $diff;
}

sub checkout {
	my $self = shift;
	my ($rev) = @_;

	return 	if (! $BINDIR);

	$rev =~ /^(\d+(?:\.\d+)+)$/		and $rev = $1
									or	return;
	my $fh = new FileHandle('-|')
				or exec("$BINDIR/co", '-q', "-p$rev", $self->{rcs_file})
				or die "exec $BINDIR/co: $!";
	my $src = join('', <$fh>);
	$fh->close;
	return $src;
}

sub delete {
	my $self = shift;
	return	if (! -f $self->{rcs_file});
	unlink($self->{rcs_file})	or die "unlink $self->{rcs_file}: $!";
	return;
}

1;
