# $Id: Folder.pm,v 1.10 2011/12/14 15:23:47 koba Exp $
#	Copyright(c) 2006-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::Resource::Folder;

use strict;
use XiuMai::Resource;
use XiuMai::RCS;
use XiuMai::XHTML::Folder;
use XiuMai::Util qw(cdata url_encode);
use DirHandle;
use vars qw($VERSION @ISA);

$VERSION = $XiuMai::VERSION;

@ISA = qw(XiuMai::Resource);

sub new {
	my $class = shift;
	my ($self) = @_;
	if ($self->{url} !~ m|/$|) {
		$self->{redirect_code} = 301;
		$self->{redirect_url}  = $self->{url}.'/';
	}
	undef $self->{type};
	undef $self->{size};
	return bless $self, $class;
}

sub file {
	my $self = shift;
	my @file = sort { $a->filename cmp $b->filename } @{$self->{file}};
	return @file;
}

sub _open {
	my $self = shift;

	my $dir = new DirHandle($self->{path})	or return;
	my @file;
	while ($_ = $dir->read) {
		next	if (/^\./);
		push(@file, new XiuMai::Resource($self->request, $_));
	}
	$dir->close;
	$self->{file} = \@file;

	return $self;
}

sub _html {
	my $self = shift;

	my ($index) = grep($_->{filename} =~ /^index(?:\.html?)?$/, $self->file);
	return $index->open		if (defined $index);

	my ($readme) = grep($_->{filename} eq 'README', $self->file);

	my $s = $self->request->param('s');

	my $html = new XiuMai::XHTML::Folder($self);
	$self->{html}
		= $html->title('folder:'.$self->request->xiumai_path_info)
			   ->as_string(
					(defined $readme
						? $readme->open->html_body.qq(<hr />\n)
						: qq(<h1>).cdata($html->title).qq(</h1>\n)),
					$html->url_navi,
					$html->file_list(s=>$s),
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub _edit {
	my $self = shift;

	my $url = cdata($self->url);

	my $s = $self->request->param('s');

	my $html = new XiuMai::XHTML::Folder($self);
	$self->{html}
		= $html->title('folder:'.$self->request->xiumai_path_info)
			   ->as_string(
					qq(<h1><a href="$url">).cdata($html->title).qq(</a></h1>\n),
					$html->url_navi,
					$html->newfile_form,
					$html->mkdir_form,
					($self->file || $self->request->xiumai_path_info eq '/'
							? '' : $html->rmdir_form),
					$html->file_list(cmd=>'edit',s=>$s),
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub update {
	my $self = shift;

	return if (! $self->w_ok);

	$self->{redirect_code} = 303;
	$self->{redirect_url}  = './';

	my $req = $self->request;
	if (defined $req->param('filename')) {
		return $self->newfile($req->param('filename'));
	}
	elsif (defined $req->param('dirname')) {
		return $self->mkdir($req->param('dirname'));
	}
	else {
		return $self->rmdir;
	}
}

sub newfile {
	my $self = shift;
	my ($filename) = @_;

	return $self if (! length $filename);

	$filename =~ s/^\./_/;
	$filename =~ s/[\:\\\/]/_/g;
	$filename =~ /^(.*)$/		and $filename = $1;

	my $fh = new FileHandle(">>$self->{path}/$filename")	or return;
	$fh->close;

	$self->{redirect_url}  = url_encode($filename).'?cmd=edit';

	return $self;
}

sub mkdir {
	my $self = shift;
	my ($dirname) = @_;

	return $self if (! length $dirname);

	$dirname =~ s/^\./_/;
	$dirname =~ s/[\:\\\/]/_/g;
	$dirname =~ /^(.*)$/		and $dirname = $1;

	mkdir("$self->{dirname}/$dirname", 0777)	or return;
	XiuMai::RCS::mkdir("$self->{dirname}/$dirname");

	$self->{redirect_url}  = "$dirname/";

	return $self;
}

sub rmdir {
	my $self = shift;
	return if ($self->request->xiumai_path_info eq '/');
	XiuMai::RCS::rmdir($self->{path});
	rmdir($self->{path})	or return;
	$self->{redirect_url}  = '../';
	return $self;
}

1;
