# $Id: HTML.pm,v 1.4 2011/12/09 15:45:42 koba Exp $
#	Copyright(c) 2008-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::Resource::HTML;

use strict;
use XiuMai::Resource::Text;
use XiuMai::Util qw(cdata);
use vars qw($VERSION @ISA);

$VERSION = $XiuMai::VERSION;

@ISA = qw(XiuMai::Resource::Text);

my $comment_regex = q/<!--(?:(?!--\s*>).)*--\s*>/;
my $tag_regex	  = q/<(?:[^"'<>]|"[^"]*"|'[^']*')*>/;

sub _strip_script {
	my ($str) = @_;
	$str =~ s/((?:(?!$comment_regex|$tag_regex).)*)
					($comment_regex)?
					($tag_regex)?
				/	_text($1)
				   .(defined $2 ? $2 : '')
				   ._tag($3)						/exgs;
	return $str;
}

sub _text {
	my ($str) = @_;
	return ''	if (! defined $str);

	$str =~ s/</&lt;/g;
	$str =~ s/>/&gt;/g;
	$str =~ s/"/&quot;/g;

	return $str;
}

sub _tag {
	my ($str) = @_;
	return ''	if (! defined $str);
	return $str	if ($str =~ /^<[\!\?]/);

	my ($s, $elem, $attr, $e) = $str =~ /^(<\/?)(\w[\w:]*)(.*?)(\/?>)$/s
											or return _text($str);
	if ($elem =~ /^SCRIPT$/i) {
		return qq(<!-- $str)		if ($s eq '<'  && $e eq '>' );
		return qq(<!-- $str -->)	if ($s eq '</' || $e eq '/>');
	}
	return qq(<!-- $str -->)		if ($elem =~ /^NOSCRIPT$/i);

	return qq(<!-- $str -->) if ($elem =~ /^(?:IFRAME|OBJECT|APPLET|EMBED)$/i);

	return $s.$elem._attr($attr).($e eq '/>' ? ' />' : '>');
}

sub _attr {
	my ($str) = @_;
	return ''	if (! defined $str);

	my $attr = '';
	foreach ($str =~ /\s+([^\s=]+(?:\s*=\s*(?:[^"'\s]+|"[^"]*"|'[^']*'))?)/gs) {

		my ($key, $value) = /^(\w[\w\:\-]*)(?:\s*=\s*(.*))?$/s	or next;
		$value = $key	if (! defined $value);
		$value =~ s/^[\"\'](.*)[\"\']$/$1/s;

		$value =~ s/&lt;/</g;
		$value =~ s/&gt;/>/g;
		$value =~ s/&quot;/"/g;
		$value =~ s/&amp;/&/g;

		next if ($key =~ /^ON/i);
		next if ($key =~ /^(?:SRC|HREF|ACTION)$/i && $value =~ /^javascript:/i);

		$attr .= qq( $key=").cdata($value).qq(");
	}

	return $attr;
}

sub _html {
	my $self = shift;

	$self->{html} = _strip_script($self->{text});
	$self->{size}  = length($self->{html});

	return $self;
}

1;
