# $Id: Text.pm,v 1.6 2011/12/14 15:23:47 koba Exp $
#	Copyright(c) 2006-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::Resource::Text;

use strict;
use XiuMai::Resource;
use XiuMai::RCS;
use XiuMai::XHTML::Text;
use XiuMai::Util qw(cdata);
use FileHandle;
use vars qw($VERSION @ISA);

$VERSION = $XiuMai::VERSION;

@ISA = qw(XiuMai::Resource);

sub new {
	my $class = shift;
	my ($self) = @_;
	$self->{RCS} = new XiuMai::RCS($self->{path});
	return bless $self, $class;
}

sub text {
	my $self = shift;
	return $self->{text};
}

sub _log {
	my $self = shift;

	my @log = $self->{RCS}->log;

	my $url   = cdata($self->url);

	my $html = new XiuMai::XHTML::Text($self);
	my $title = cdata($html->msg('log.title', $self->filename));
	$self->{html}
		= $html->title($title)
			   ->as_string(
					qq(<h1><a href="$url">).cdata($html->title).qq(</a></h1>\n),
					$html->url_navi,
					$html->log(@log),
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub _diff {
	my $self = shift;

	my $diff = $self->{RCS}->diff($self->request->param('rev'));

	my $url   = cdata($self->url);

	my $html = new XiuMai::XHTML::Text($self);
	my $title = cdata($html->msg('diff.title', $self->filename));
	$self->{html}
		= $html->title($title)
			   ->as_string(
					qq(<h1><a href="$url">).cdata($html->title).qq(</a></h1>\n),
					$html->url_navi,
					$html->diff($diff),
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub _open {
	my $self = shift;

	if (! $self->{ignore_rev} && $self->request->param('rev')) {
		$self->{text} = $self->{RCS}->checkout($self->request->param('rev'));
		return $self	if (defined $self->{text});
	}

	my $fh = new FileHandle($self->{path})	or return;
	$self->{text} = '';
	while (<$fh>) {
		$self->{text} .= $_;
	}
	return $self;
}

sub _edit {
	my $self = shift;

	my $url = cdata($self->url);

	my $html = new XiuMai::XHTML::Text($self);
	$self->{html}
		= $html->title($self->filename)
			   ->as_string(
					qq(<h1><a href="$url">).cdata($html->title).qq(</a></h1>\n),
					$html->url_navi,
					$html->text_form,
				);

	$self->{type}  = undef;
	$self->{size}  = length($self->{html});
	$self->{mtime} = undef;
	$self->{lang}  = [$html->content_language];

	return $self;
}

sub update {
	my $self = shift;

	return if (! $self->w_ok);

	my $text = $self->request->param('text');

	$self->{redirect_code} = 303;
	$self->{redirect_url}  = './';

	if (! defined $text || ! length $text) {
		$self->{RCS}->delete;
		unlink($self->{path})	or return;
		return $self;
	}

	my $log = $self->{RCS}->log;
	if (defined $log
		&& ($self->mtime + 24*60*60 < time()
			|| $log->{locker} ne $self->request->xiumai_user))
	{
		$self->{RCS}->checkin($log->{locker}, $self->request->xiumai_user);
	}

	chomp $text; $text .= "\n";

	my $fh = new FileHandle(">$self->{path}")	or return;
	print $fh $text;
	$fh->close;

	if (! defined $log) {
		XiuMai::RCS::mkdir($self->{dirname});
		$self->{RCS}->checkin($self->request->xiumai_user);
	}

	return $self;
}

1;
