# $Id: Folder.pm,v 1.5 2011/12/17 13:34:03 koba Exp $
#	Copyright(c) 2006-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::XHTML::Folder;

use strict;
use XiuMai::XHTML::Resource;
use XiuMai::Util qw(cdata url_encode date_str size_str);
use vars qw($VERSION @ISA);

$VERSION = $XiuMai::VERSION;

@ISA = qw(XiuMai::XHTML::Resource);

sub file_list {
	my $self = shift;
	my %param = @_;

	my $edit_mode = defined $param{cmd} && $param{cmd} eq 'edit';
	my $sort	  = defined $param{s} ? $param{s} : 'n';

	my @param = map {
					my $s = $_ eq $sort ? uc($_) : $_;
					$edit_mode ? "?cmd=edit&s=$s" : "?s=$s";
				} ('n','m','s','d');
	my @class = map {
					  $sort eq $_     ? qq( class="asc")
					: $sort eq uc($_) ? qq( class="desc")
					: '';
				} ('n','m','s','d');

	my @file = sort {
					local $^W = undef;
						$sort eq 'n' and $a->filename cmp $b->filename
					or	$sort eq 'N' and $b->filename cmp $a->filename
					or	$sort eq 'm' and $a->mtime	  <=> $b->mtime
					or	$sort eq 'M' and $b->mtime	  <=> $a->mtime
					or	$sort eq 's' and $a->size	  <=> $b->size
					or	$sort eq 'S' and $b->size	  <=> $a->size
					or	$sort eq 'd' and $a->type	  cmp $b->type
					or	$sort eq 'D' and $b->type	  cmp $a->type
				} $self->{Resource}->file;

	my $html = qq(<table class="file_list" style="width: 100%;">\n);

	my $name		= cdata($self->msg('folder.file_list.name.label'));
	my $mtime		= cdata($self->msg('folder.file_list.mtime.label'));
	my $size		= cdata($self->msg('folder.file_list.size.label'));
	my $description	= cdata($self->msg('folder.file_list.description.label'));
	my $edit		= cdata($self->msg('folder.file_list.edit.label'));
	$html .= qq(\t<tr>)
		   . qq(<th$class[0]><a href="$param[0]">$name</a></th>)
		   . qq(<th$class[1]><a href="$param[1]">$mtime</a></th>)
		   . qq(<th$class[2]><a href="$param[2]">$size</a></th>)
		   . qq(<th$class[3]><a href="$param[3]">$description</a></th>)
		   . ($edit_mode ? qq(<th></th>) : '')
		   . qq(</tr>\n);

	foreach my $file (@file) {
		next if (! $edit_mode && $file->filename eq 'README');
		my $name		= cdata($file->filename);
		my $url			= cdata(url_encode($file->filename));
		my $mtime		= cdata(date_str($file->mtime));
		my $size		= cdata(size_str($file->size));
		my $description	= cdata($file->type);
		my $edit_url	= $url.'?cmd=edit';
		if (! $file->type) {
			$name .= '/';
			$url  .= '/';
			undef $edit_url;
		}

		$html .= qq(\t<tr>)
			   . qq(<td><a href="$url">$name</a></td>)
			   . qq(<td style="text-align: right;">$mtime</td>)
			   . qq(<td style="text-align: right;">$size</td>)
			   . qq(<td>$description</td>)
			   . ($edit_mode
			   		? (defined $edit_url
			   				? qq(<td><a href="$edit_url">$edit</a></td>)
							: qq(<td></td>))
			   		: '')
			   . qq(</tr>\n);
	}

	$html .= qq(</table>\n);
	return $html;
}

sub newfile_form {
	my $self = shift;

	my $url      = cdata($self->{Resource}->url);
	my $legend   = cdata($self->msg('folder.newfile_form.label'));
	my $filename = cdata($self->msg('folder.newfile_form.filename.label'));
	my $submit   = cdata($self->msg('folder.newfile_form.submit.label'));
	my $session_id = cdata($self->session_id);

	return <<"++";
<form class="newfile_form" method="post" action="$url">
<fieldset>
	<legend>$legend</legend>
	$filename
	<input name="session_id" type="hidden" value="$session_id" />
	<input name="filename" />
	<input type="submit" value="$submit" />
</fieldset>
</form>
++
}

sub mkdir_form {
	my $self = shift;

	my $url     = cdata($self->{Resource}->url);
	my $legend  = cdata($self->msg('folder.mkdir_form.label'));
	my $dirname = cdata($self->msg('folder.mkdir_form.dirname.label'));
	my $submit  = cdata($self->msg('folder.mkdir_form.submit.label'));
	my $session_id = cdata($self->session_id);

	return <<"++";
<form class="mkdir_form" method="post" action="$url">
<fieldset>
	<legend>$legend</legend>
	$dirname
	<input name="session_id" type="hidden" value="$session_id" />
	<input name="dirname" />
	<input type="submit" value="$submit" />
</fieldset>
</form>
++
}

sub rmdir_form {
	my $self = shift;

	my $url     = cdata($self->{Resource}->url);
	my $legend  = cdata($self->msg('folder.rmdir_form.label'));
	my $submit  = cdata($self->msg('folder.rmdir_form.submit.label'));
	my $session_id = cdata($self->session_id);

	return <<"++";
<form method="post" action="$url">
<fieldset>
	<legend>$legend</legend>
	<input name="session_id" type="hidden" value="$session_id" />
	<input type="submit" value="$submit" />
</fieldset>
</form>
++
}

1;
