# $Id: Resource.pm,v 1.4 2008/04/05 00:40:05 koba Exp $
#	Copyright(c) 2006 Satoshi Kobayashi, All rights reserved.

package XiuMai::XHTML::Resource;

use strict;
use XiuMai;
use XiuMai::XHTML;
use XiuMai::Util qw(cdata url_encode url_decode);
use vars qw($VERSION @ISA);

$VERSION = $XiuMai::VERSION;

@ISA = qw(XiuMai::XHTML);

sub new {
	my $class = shift;
	my ($r) = @_;
	my $self = new XiuMai::XHTML($r->request);
	bless $self, $class;

	$self->{Resource} = $r;
	$self->accept_language($r->request->accept_language);

	return $self;
}

sub session_id {
	my $self = shift;
	return $self->{Resource}->request->xiumai_session_id;
}

sub url_navi {
	my $self = shift;

	my $r = $self->{Resource};
	my $base_url  = $r->request->xiumai_base_url;
	my $path_info = $r->request->xiumai_path_info;
	$path_info =~ s|/[^/]*?$||;
	$path_info =~ s|^/||;
	my @path = split('/', $path_info);

	my $html = qq(<p class="url_navi">);
	my $url = cdata("$base_url/");
	$html .= qq(<a href="$url">TOP</a>/);
	foreach my $path (@path) {
		$url .= cdata(url_encode($path)).'/';
		$path = cdata($path);
		$html .= qq(<a href="$url">$path</a>/);
	}
	$html .= qq(</p>\n);

	return $html;
}

sub upload_form {
	my $self = shift;

	my $url    = cdata($self->{Resource}->url);
	my $legend = cdata($self->msg('resource.upload_form.label'));
	my $submit = cdata($self->msg('resource.upload_form.submit.label'));
	my $session_id = cdata($self->session_id);

	return <<"++";
<form method="post" action="$url" enctype="multipart/form-data">
<fieldset>
	<legend>$legend</legend>
	<input name="session_id" type="hidden" value="$session_id" />
	<input name="file" type="file" size="40" />
	<input type="submit" value="$submit" />
</fieldset>
</form>
++
}

sub _toolbar {
	my $self = shift;

	if ($self->{Resource}->w_ok) {
		my $label = cdata($self->msg('toolbar.edit.label'));
		push(@{$self->{toolbar}},
			qq(<li><a href="?cmd=edit" accesskey="e">$label</a></li>));
	}
	return $self->SUPER::_toolbar;
}

1;
