# $Id: Text.pm,v 1.4 2011/12/17 13:27:54 koba Exp $
#	Copyright(c) 2006-2011 Satoshi Kobayashi, All rights reserved.

package XiuMai::XHTML::Text;

use strict;
use XiuMai;
use XiuMai::XHTML::Resource;
use XiuMai::RCS;
use XiuMai::Util qw(cdata date_str);
use FileHandle;
use vars qw($VERSION @ISA);

$VERSION = $XiuMai::VERSION;

@ISA = qw(XiuMai::XHTML::Resource);

sub text_form {
	my $self = shift;

	my $url    = cdata($self->{Resource}->url);
	my $text   = cdata($self->{Resource}->text);
	my $submit = cdata($self->msg('text.text_form.submit.label'));
	my $session_id = cdata($self->session_id);

	return <<"++";
<form method="post" action="$url">
<p><input name="session_id" type="hidden" value="$session_id" />
<textarea name="text" rows="32" cols="88">$text</textarea></p>
<p><input type="submit" value="$submit" /></p>
</form>
++
}

sub log {
	my $self = shift;
	my (@log) = @_;

	return ''	if (! @log);

	my $html = qq(<table class="log">\n);

	my $rev		= cdata($self->msg('log.revision.label'));
	my $mtime	= cdata($self->msg('log.mtime.label'));
	my $author	= cdata($self->msg('log.author.label'));
	$html .= qq(<tr>\n)
		   . qq(\t<th>$rev</th>\n)
		   . qq(\t<th>$mtime</th>\n)
		   . qq(\t<th>$author</th>\n)
		   . qq(\t<th></th>\n)
		   . qq(\t<th></th>\n)
		   . qq(\t<th></th>\n)
		   . qq(\t<th></th>\n)
		   . qq(</tr>\n);

	my $url		= $self->{Resource}->url;
	my $diff1	= cdata($self->msg('log.diff1.label'));
	my $diff2	= cdata($self->msg('log.diff2.label'));
	my $show	= cdata($self->msg('log.show.label'));
	my $edit	= cdata($self->msg('log.edit.label'));

	for (my $i = 0; $i < @log; $i++) {
		my $rev		= cdata($log[$i]{rev});
		my $mtime	= cdata(date_str($log[$i]{mtime}));
		my $author	= cdata($log[$i]{author});
		my $diff1_url	= ($i + 1 < @log)
							&& cdata("$url?cmd=diff"
									."&rev=$log[$i+1]{rev}&rev=$log[$i]{rev}");
		my $diff2_url	= cdata("$url?cmd=diff&rev=$log[$i]{rev}");
		my $show_url	= cdata("$url?rev=$log[$i]{rev}");
		my $edit_url	= cdata("$url?cmd=edit&rev=$log[$i]{rev}");

		$html .= qq(<tr>\n)
			   . qq(\t<td style="text-align: right;">$rev</td>\n)
			   . qq(\t<td style="text-align: right;">$mtime</td>\n)
			   . qq(\t<td>$author</td>\n)
			   . ($diff1_url ? qq(\t<td><a href="$diff1_url">$diff1</a></td>\n)
			   				 : qq(\t<td></td>\n))
			   . qq(\t<td><a href="$diff2_url">$diff2</a></td>\n)
			   . qq(\t<td><a href="$show_url">$show</a></td>\n)
			   . qq(\t<td><a href="$edit_url">$edit</a></td>\n)
			   . qq(</tr>\n);
	}

	$html .= qq(</table>\n);

	return $html;
}

sub diff {
	my $self = shift;
	my ($diff) = @_;

	my $html = qq(<pre>).cdata($diff).qq(</pre>\n);
	return $html;
}

sub _toolbar {
	my $self = shift;

	return $self->SUPER::_toolbar	if (! $XiuMai::RCS::BINDIR);

	if ($self->{Resource}->w_ok) {
		my $label = cdata($self->msg('toolbar.log.label'));
		my $url   = cdata($self->{Resource}->url);
		push(@{$self->{toolbar}},
			qq(<li><a href="$url?cmd=log">$label</a></li>));
	}
	if ($self->{Resource}->w_ok && $self->{Resource}->_cmd ne 'diff') {
		my $label = cdata($self->msg('toolbar.edit.label'));
		my $rev   = cdata($self->{Request}->param('rev'));
		if ($rev) {
			push(@{$self->{toolbar}},
				qq(<li><a href="?cmd=edit&rev=$rev" accesskey="e">)
				.qq($label</a></li>));
		}
		else {
			push(@{$self->{toolbar}},
				qq(<li><a href="?cmd=edit" accesskey="e">$label</a></li>));
		}
	}
	return XiuMai::XHTML::_toolbar($self);
}

1;
